@@ -7,12 +7,12 @@
  *	attributes.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkFrame.c,v 1.2 1998/09/14 18:23:10 stanton Exp $
+ * RCS: @(#) $Id: tkFrame.c,v 1.3 1999/04/16 01:51:14 stanton Exp $
  */
 
 #include "default.h"
@@ -105,14 +105,14 @@ typedef struct {
  */
 
 static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc,
+    (Tk_OptionParseProc *) TkTileParseProc,
+    TkTilePrintProc,
     (ClientData) NULL
 };
 
 static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
+    (Tk_OptionParseProc *) TkOffsetParseProc,
+    TkOffsetPrintProc,
     (ClientData) NULL
 };
 
@@ -202,9 +202,6 @@ static void		FrameEventProc _ANSI_ARGS_((ClientData clientData,
 static int		FrameWidgetCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		MapFrame _ANSI_ARGS_((ClientData clientData));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
-
 
 /*
  *--------------------------------------------------------------
@@ -473,7 +470,7 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     if (toplevel) {
 	Tcl_DoWhenIdle(MapFrame, (ClientData) framePtr);
     }
-    interp->result = Tk_PathName(new);
+    Tcl_SetResult(interp, Tk_PathName(new), TCL_STATIC);
     return TCL_OK;
 
     error:
@@ -657,7 +654,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message.
+ *	returned, then the interp's result contains an error message.
  *
  * Side effects:
  *	Configuration information, such as text string, colors, font,
