@@ -6,12 +6,12 @@
  *	widgets.
  *
  * Copyright (c) 1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkImage.c,v 1.2 1998/09/14 18:23:12 stanton Exp $
+ * RCS: @(#) $Id: tkImage.c,v 1.3 1999/04/16 01:51:15 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -69,9 +69,6 @@ static Blt_ListItem *Blt_NewItem _ANSI_ARGS_((char *key));
 static void Blt_LinkAfter _ANSI_ARGS_((Blt_List *listPtr,
 	Blt_ListItem *itemPtr, Blt_ListItem *afterPtr));
 static void Blt_FreeItem _ANSI_ARGS_((Blt_ListItem *itemPtr));
-static void TileChangedProc _ANSI_ARGS_((ClientData clientData,
-	int x, int y, int width, int height, int imageWidth,
-	int imageHeight));
 
 #define Blt_FirstListItem(listPtr)	((listPtr)->headPtr)
 #define Blt_NextItem(itemPtr) 		((itemPtr)->nextPtr)
@@ -135,12 +132,11 @@ typedef struct ImageMaster {
 				 * derived from this name. */
 } ImageMaster;
 
-/*
- * The following variable points to the first in a list of all known
- * image types.
- */
-
-static Tk_ImageType *imageTypeList = NULL;
+typedef struct ThreadSpecificData {
+    Tk_ImageType *imageTypeList;/* First in a list of all known image 
+				 * types. */  
+} ThreadSpecificData;           
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Prototypes for local procedures:
@@ -175,8 +171,11 @@ Tk_CreateImageType(typePtr)
 				 * in by caller.  Must not have been passed
 				 * to Tk_CreateImageType previously. */
 {
-    typePtr->nextPtr = imageTypeList;
-    imageTypeList = typePtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+
+    typePtr->nextPtr = tsdPtr->imageTypeList;
+    tsdPtr->imageTypeList = typePtr;
 }
 
 /*
@@ -211,8 +210,10 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
     Image *imagePtr;
     Tcl_HashEntry *hPtr;
     Tcl_HashSearch search;
-    char idString[30], *name;
-    static int id = 0;
+    char idString[16 + TCL_INTEGER_SPACE], *name;
+    TkDisplay *dispPtr = winPtr->dispPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     static char **argv = NULL;
 
     if (argc < 2) {
@@ -242,7 +243,7 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
 	 * Look up the image type.
 	 */
 
-	for (typePtr = imageTypeList; typePtr != NULL;
+	for (typePtr = tsdPtr->imageTypeList; typePtr != NULL;
 		typePtr = typePtr->nextPtr) {
 	    if ((c == typePtr->name[0])
 		    && (strcmp(argv[2], typePtr->name) == 0)) {
@@ -260,8 +261,8 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
 	 */
 
 	if ((argc == 3) || (argv[3][0] == '-')) {
-	    id++;
-	    sprintf(idString, "image%d", id);
+	    dispPtr->imageId++;
+	    sprintf(idString, "image%d", dispPtr->imageId);
 	    name = idString;
 	    firstOption = 3;
 	} else {
@@ -329,7 +330,9 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
 		masterPtr->height);
 	   }
 	}
-	Tcl_AppendResult(interp, Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr), (char *) NULL);
+	Tcl_SetResult(interp,
+		Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr),
+		TCL_STATIC);
     } else if ((c == 'd') && (strncmp(argv[1], "delete", length) == 0)) {
 	for (i = 2; i < argc; i++) {
 	    hPtr = Tcl_FindHashEntry(&winPtr->mainPtr->imageTable, argv[i]);
@@ -342,6 +345,7 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
 	    DeleteImage(masterPtr);
 	}
     } else if ((c == 'h') && (strncmp(argv[1], "height", length) == 0)) {
+	
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
 		    " height name\"", (char *) NULL);
@@ -380,7 +384,7 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
 	}
 	masterPtr = (ImageMaster *) Tcl_GetHashValue(hPtr);
 	if (masterPtr->typePtr != NULL) {
-	    Tcl_AppendResult(interp, masterPtr->typePtr->name, (char *) NULL);
+	    Tcl_SetResult(interp, masterPtr->typePtr->name, TCL_STATIC);
 	}
     } else if ((c == 't') && (strcmp(argv[1], "types") == 0)) {
 	if (argc != 2) {
@@ -388,11 +392,12 @@ Tk_ImageObjCmd(clientData, interp, argc, objv)
 		    " types\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
-	for (typePtr = imageTypeList; typePtr != NULL;
+	for (typePtr = tsdPtr->imageTypeList; typePtr != NULL;
 		typePtr = typePtr->nextPtr) {
 	    Tcl_AppendElement(interp, typePtr->name);
 	}
     } else if ((c == 'w') && (strncmp(argv[1], "width", length) == 0)) {
+
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
 		    " width name\"", (char *) NULL);
@@ -502,7 +507,7 @@ Tk_NameOfImage(imageMaster)
  * Results:
  *	The return value is a token for the image.  If there is no image
  *	by the given name, then NULL is returned and an error message is
- *	left in interp->result.
+ *	left in the interp's result.
  *
  * Side effects:
  *	Tk records the fact that the widget is using the image, and
