@@ -29,7 +29,7 @@
  * |   provided "as is" without express or implied warranty.           |
  * +-------------------------------------------------------------------+
  *
- * SCCS: @(#) tkImgGIF.c 1.19 97/08/13 15:23:45
+ * RCS: @(#) $Id: tkImgGIF.c,v 1.3 1999/04/16 01:51:15 stanton Exp $
  */
 
 /*
@@ -62,6 +62,18 @@ typedef struct mFile {
 #include "tkInt.h"
 #include "tkPort.h"
 
+/*
+ * 			 HACK ALERT!!  HACK ALERT!!  HACK ALERT!!
+ * This code is hard-wired for reading from files.  In order to read
+ * from a data stream, we'll trick fread so we can reuse the same code.
+ * 0==from file; 1==from base64 encoded data; 2==from binary data
+ */
+
+typedef struct ThreadSpecificData {
+    int fromData;
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
+
 /*
  * The format record for the GIF file format:
  */
@@ -101,16 +113,7 @@ Tk_PhotoImageFormat tkImgFmtGIF = {
 #define CM_ALPHA		3
 #define MAX_LWZ_BITS		12
 #define LM_to_uint(a,b)         (((b)<<8)|(a))
-#define ReadOK(file,buffer,len)	(Fread(buffer, 1, len, file) >= 0)
-
-/*
- * 			 HACK ALERT!!  HACK ALERT!!  HACK ALERT!!
- * This code is hard-wired for reading from files.  In order to read
- * from a data stream, we'll trick fread so we can reuse the same code.
- * 0==from file; 1==from base64 encoded data; 2==from binary data
- */
- 
-static int fromData = 0;
+#define ReadOK(file,buffer,len)	(Fread(buffer, len, 1, file) != 0)
 
 /*
  * Prototypes for local procedures defined in this file:
@@ -190,7 +193,7 @@ FileMatchGIF(chan, fileName, format, widthPtr, heightPtr, interp)
  *
  * Results:
  *	A standard TCL completion code.  If TCL_ERROR is returned
- *	then an error message is left in interp->result.
+ *	then an error message is left in the interp's result.
  *
  * Side effects:
  *	The access position in file f is changed, and new data is
@@ -287,6 +290,7 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
     block.offset[0] = 0;
     block.offset[1] = 1;
     block.offset[2] = 2;
+    block.offset[3] = 0;
     block.pixelPtr = NULL;
 
     while (1) {
@@ -315,12 +319,14 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	     */
 
 	    if (Fread(buf, 1, 1, chan) != 1) {
-		interp->result =
-			"error reading extension function code in GIF image";
+		Tcl_SetResult(interp,
+			"error reading extension function code in GIF image",
+			TCL_STATIC);
 		goto error;
 	    }
 	    if (DoExtension(chan, buf[0], &transparent) < 0) {
-		interp->result = "error reading extension in GIF image";
+		Tcl_SetResult(interp, "error reading extension in GIF image",
+			TCL_STATIC);
 		goto error;
 	    }
 	    continue;
@@ -334,7 +340,9 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	}
 
 	if (Fread(buf, 1, 9, chan) != 9) {
-	    interp->result = "couldn't read left/top/width/height in GIF image";
+	    Tcl_SetResult(interp,
+		    "couldn't read left/top/width/height in GIF image",
+		    TCL_STATIC);
 	    goto error;
 	}
 
@@ -471,7 +479,7 @@ StringMatchGIF(dataObj, format, widthPtr, heightPtr, interp)
     int got, length;
     MFile handle;
 
-    data = Tcl_GetStringFromObj(dataObj, &length);
+    data = Tcl_GetByteArrayFromObj(dataObj, &length);
 
     /* Header is a minimum of 10 bytes */
     if (length < 10) {
@@ -480,14 +488,14 @@ StringMatchGIF(dataObj, format, widthPtr, heightPtr, interp)
 
     /* Check whether the data is Base64 encoded */
 
-    if ((strncmp("GIF87a", data, 6) != 0) && 
-	(strncmp("GIF89a", data, 6) != 0)) {
+    if ((strncmp("\107\111\106\70\67\141", data, 6) != 0) && 
+	(strncmp("\107\111\106\70\71\141", data, 6) != 0)) {
       /* Try interpreting the data as Base64 encoded */
       mInit((unsigned char *) data, &handle);
       got = Mread(header, 10, 1, &handle);
       if (got != 10
-	      || ((strncmp("GIF87a", (char *) header, 6) != 0)
-	      && (strncmp("GIF89a", (char *) header, 6) != 0))) {
+	      || ((strncmp("\107\111\106\70\67\141", (char *) header, 6) != 0)
+	      && (strncmp("\107\111\106\70\71\141", (char *) header, 6) != 0))) {
 	  return 0;
       }
     } else {
@@ -509,7 +517,7 @@ StringMatchGIF(dataObj, format, widthPtr, heightPtr, interp)
  *
  * Results:
  *	A standard TCL completion code.  If TCL_ERROR is returned
- *	then an error message is left in interp->result.
+ *	then an error message is left in the interp's result.
  *
  * Side effects:
  *	new data is added to the image given by imageHandle.  This
@@ -530,27 +538,29 @@ StringReadGIF(interp, dataObj, format, imageHandle,
     int  width, height;		/*   image to copy */
     int srcX, srcY;
 {
-	int result;
-	MFile handle;
-	Tcl_Channel dataSrc;
-	char *data;
-	/* Check whether the data is Base64 encoded */
-	data = Tcl_GetStringFromObj(dataObj, NULL);
-	if ((strncmp("GIF87a", data, 6) != 0) && 
-	    (strncmp("GIF89a", data, 6) != 0)) {
-	  mInit((unsigned char *)data,&handle);
-	  fromData = 1;
-	  dataSrc = (Tcl_Channel) &handle;
-	} else {
-	  fromData = 2;
-	  mInit((unsigned char *)data,&handle);
-	  dataSrc = (Tcl_Channel) &handle;
-	}
-	result = FileReadGIF(interp, dataSrc, "inline data",
-		format, imageHandle, destX, destY, width, height,
-		srcX, srcY);
-	fromData = 0;
-	return(result);
+    int result;
+    MFile handle;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    Tcl_Channel dataSrc;
+    char *data;
+    /* Check whether the data is Base64 encoded */
+    data = Tcl_GetByteArrayFromObj(dataObj, NULL);
+    if ((strncmp("\107\111\106\70\67\141", data, 6) != 0) && 
+	    (strncmp("\107\111\106\70\71\141", data, 6) != 0)) {
+	mInit((unsigned char *)data,&handle);
+	tsdPtr->fromData = 1;
+	dataSrc = (Tcl_Channel) &handle;
+    } else {
+	tsdPtr->fromData = 2;
+	mInit((unsigned char *)data,&handle);
+	dataSrc = (Tcl_Channel) &handle;
+    }
+    result = FileReadGIF(interp, dataSrc, "inline data",
+	    format, imageHandle, destX, destY, width, height,
+            srcX, srcY);
+    tsdPtr->fromData = 0;
+    return(result);
 }
 
 /*
@@ -582,8 +592,8 @@ ReadGIFHeader(chan, widthPtr, heightPtr)
     unsigned char buf[7];
 
     if ((Fread(buf, 1, 6, chan) != 6)
-	    || ((strncmp("GIF87a", (char *) buf, 6) != 0)
-	    && (strncmp("GIF89a", (char *) buf, 6) != 0))) {
+	    || ((strncmp("\107\111\106\70\67\141", (char *) buf, 6) != 0)
+	    && (strncmp("\107\111\106\70\71\141", (char *) buf, 6) != 0))) {
 	return 0;
     }
 
@@ -724,7 +734,7 @@ ReadImage(interp, imagePtr, chan, len, rows, cmap,
     }
 
     if (LWZReadByte(chan, 1, c) < 0) {
-	interp->result = "format error in GIF image";
+	Tcl_SetResult(interp, "format error in GIF image", TCL_STATIC);
 	return TCL_ERROR;
     }
 
@@ -1158,8 +1168,11 @@ Fread(dst, hunk, count, chan)
     size_t hunk,count;		/* how many */
     Tcl_Channel chan;
 {
-  MFile *handle;
-    switch (fromData) {
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    MFile *handle;
+
+    switch (tsdPtr->fromData) {
       case 1:
 	return(Mread(dst, hunk, count, (MFile *) chan));
       case 2:
