@@ -13,7 +13,7 @@
  *	   Department of Computer Science,
  *	   Australian National University.
  *
- * RCS: @(#) $Id: tkImgPPM.c,v 1.2 1998/09/14 18:23:13 stanton Exp $
+ * RCS: @(#) $Id: tkImgPPM.c,v 1.4 1999/04/24 01:50:47 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -111,7 +111,7 @@ FileMatchPPM(chan, fileName, formatString, widthPtr, heightPtr, interp)
  *
  * Results:
  *	A standard TCL completion code.  If TCL_ERROR is returned
- *	then an error message is left in interp->result.
+ *	then an error message is left in the interp's result.
  *
  * Side effects:
  *	The access position in file f is changed, and new data is
@@ -152,7 +152,7 @@ FileReadPPM(interp, chan, fileName, formatString, imageHandle, destX, destY,
 	return TCL_ERROR;
     }
     if ((maxIntensity <= 0) || (maxIntensity >= 256)) {
-	char buffer[30];
+	char buffer[TCL_INTEGER_SPACE];
 
 	sprintf(buffer, "%d", maxIntensity);
 	Tcl_AppendResult(interp, "PPM image file \"", fileName,
@@ -172,6 +172,7 @@ FileReadPPM(interp, chan, fileName, formatString, imageHandle, destX, destY,
 	return TCL_OK;
     }
 
+    block.offset[3] = 0;
     if (type == PGM) {
         block.pixelSize = 1;
         block.offset[0] = 0;
@@ -245,7 +246,7 @@ FileReadPPM(interp, chan, fileName, formatString, imageHandle, destX, destY,
  *
  * Results:
  *	A standard TCL completion code.  If TCL_ERROR is returned
- *	then an error message is left in interp->result.
+ *	then an error message is left in the interp's result.
  *
  * Side effects:
  *	Data is written to the file given by "fileName".
@@ -264,13 +265,22 @@ FileWritePPM(interp, fileName, formatString, blockPtr)
     int w, h;
     int greenOffset, blueOffset, nBytes;
     unsigned char *pixelPtr, *pixLinePtr;
-    char header[30];
+    char header[16 + TCL_INTEGER_SPACE * 2];
 
     chan = Tcl_OpenFileChannel(interp, fileName, "w", 0666);
     if (chan == NULL) {
 	return TCL_ERROR;
     }
 
+    if (Tcl_SetChannelOption(interp, chan, "-translation", "binary")
+	    != TCL_OK) {
+	return TCL_ERROR;
+    }
+    if (Tcl_SetChannelOption(interp, chan, "-encoding", "binary")
+	    != TCL_OK) {
+	return TCL_ERROR;
+    }
+    
     sprintf(header, "P6\n%d %d\n255\n", blockPtr->width, blockPtr->height);
     Tcl_Write(chan, header, -1);
 	
