@@ -6,12 +6,12 @@
  *	in a window according to a particular aspect ratio.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMessage.c,v 1.2 1998/09/14 18:23:15 stanton Exp $
+ * RCS: @(#) $Id: tkMessage.c,v 1.3 1999/04/16 01:51:20 stanton Exp $
  */
 
 #include "tkPort.h"
@@ -40,7 +40,7 @@ typedef struct {
 
     char *string;		/* String displayed in message. */
     int numChars;		/* Number of characters in string, not
-				 * including terminating NULL character. */
+				 * including terminating NULL. */
     char *textVarName;		/* Name of variable (malloc'ed) or NULL.
 				 * If non-NULL, message displays the contents
 				 * of this variable. */
@@ -107,8 +107,8 @@ typedef struct {
  */
 
 static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc,
+    (Tk_OptionParseProc *) TkTileParseProc,
+    TkTilePrintProc,
     (ClientData) NULL
 };
 
@@ -194,8 +194,6 @@ static int		ConfigureMessage _ANSI_ARGS_((Tcl_Interp *interp,
 			    int flags));
 static void		DestroyMessage _ANSI_ARGS_((char *memPtr));
 static void		DisplayMessage _ANSI_ARGS_((ClientData clientData));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
 
 /*
  * The structure below defines message class behavior by means of procedures
@@ -283,7 +281,7 @@ Tk_MessageCmd(clientData, interp, argc, argv)
     msgPtr->tile = NULL;
     msgPtr->tileGC = NULL;
 
-    TkClassOption(msgPtr->tkwin, "Message",&argc,&argv);
+    TkClassOption(msgPtr->tkwin, "Message", &argc, &argv);
     TkSetClassProcs(msgPtr->tkwin, &messageClass, (ClientData) msgPtr);
     Tk_CreateEventHandler(msgPtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
@@ -292,7 +290,7 @@ Tk_MessageCmd(clientData, interp, argc, argv)
 	goto error;
     }
 
-    interp->result = Tk_PathName(msgPtr->tkwin);
+    Tcl_SetResult(interp, Tk_PathName(msgPtr->tkwin), TCL_STATIC);
     return TCL_OK;
 
     error:
@@ -441,7 +439,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message.
+ *	returned, then the interp's result contains an error message.
  *
  * Side effects:
  *	Configuration information, such as text string, colors, font,
@@ -505,7 +503,7 @@ ConfigureMessage(interp, msgPtr, argc, argv, flags)
      * that couldn't be specified to Tk_ConfigureWidget.
      */
 
-    msgPtr->numChars = strlen(msgPtr->string);
+    msgPtr->numChars = Tcl_NumUtfChars(msgPtr->string, -1);
 
     if (msgPtr->highlightWidth < 0) {
 	msgPtr->highlightWidth = 0;
@@ -908,8 +906,8 @@ MessageTextVarProc(clientData, interp, name1, name2, flags)
     if (msgPtr->string != NULL) {
 	ckfree(msgPtr->string);
     }
-    msgPtr->numChars = strlen(value);
-    msgPtr->string = (char *) ckalloc((unsigned) (msgPtr->numChars + 1));
+    msgPtr->numChars = Tcl_NumUtfChars(value, -1);
+    msgPtr->string = (char *) ckalloc((unsigned) (strlen(value) + 1));
     strcpy(msgPtr->string, value);
     ComputeMessageGeometry(msgPtr);
 
