@@ -9,11 +9,12 @@
  *
  * Copyright (c) 1992-1994 The Regents of the University of California.
  * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1999 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkText.c,v 1.2 1998/09/14 18:23:17 stanton Exp $
+ * RCS: @(#) $Id: tkText.c,v 1.7 1999/04/21 21:53:28 rjohnson Exp $
  */
 
 #include "default.h"
@@ -32,20 +33,20 @@
  */
 
 static Tk_CustomOption stateOption = {
-    (Tk_OptionParseProc *) Tk_StateParseProc,
-    Tk_StatePrintProc,
+    (Tk_OptionParseProc *) TkStateParseProc,
+    TkStatePrintProc,
     (ClientData) NULL	/* only "normal" and "disabled" */
 };
 
 static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc,
+    (Tk_OptionParseProc *) TkTileParseProc,
+    TkTilePrintProc,
     (ClientData) NULL
 };
 
 static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
+    (Tk_OptionParseProc *) TkOffsetParseProc,
+    TkOffsetPrintProc,
     (ClientData) NULL
 };
 
@@ -186,7 +187,6 @@ Tk_CustomOption textWrapModeOption = {
     (ClientData) NULL
 };
 
-
 /*
  *--------------------------------------------------------------
  *
@@ -240,14 +240,13 @@ WrapModeParseProc(clientData, interp, tkwin, value, widgRec, offset)
 	*wrapPtr = TEXT_WRAPMODE_WORD;
 	return TCL_OK;
     }
-
     Tcl_AppendResult(interp, "bad wrap mode \"", value,
 	    "\": must be char, none, or word",
 	    (char *) NULL);
     *wrapPtr = TEXT_WRAPMODE_CHAR;
     return TCL_ERROR;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -292,7 +291,7 @@ WrapModePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 	return "";
     }
 }
-
+
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -325,8 +324,6 @@ static void		DumpLine _ANSI_ARGS_((Tcl_Interp *interp,
 static int		DumpSegment _ANSI_ARGS_((Tcl_Interp *interp, char *key,
 			    char *value, char * command, int lineno, int offset,
 			    int what));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
 
 /*
  * The structure below defines text class behavior by means of procedures
@@ -423,7 +420,7 @@ Tk_TextCmd(clientData, interp, argc, argv)
     textPtr->prevWidth = Tk_Width(new);
     textPtr->prevHeight = Tk_Height(new);
     TkTextCreateDInfo(textPtr);
-    TkTextMakeIndex(textPtr->tree, 0, 0, &startIndex);
+    TkTextMakeByteIndex(textPtr->tree, 0, 0, &startIndex);
     TkTextSetYView(textPtr, &startIndex, 0);
     textPtr->selTagPtr = NULL;
     textPtr->selBorder = NULL;
@@ -461,13 +458,14 @@ Tk_TextCmd(clientData, interp, argc, argv)
      */
 
     textPtr->selTagPtr = TkTextCreateTag(textPtr, "sel");
-    textPtr->selTagPtr->reliefString = (char *) ckalloc(7);
+    textPtr->selTagPtr->reliefString =
+	    (char *) ckalloc(sizeof(DEF_TEXT_SELECT_RELIEF));
     strcpy(textPtr->selTagPtr->reliefString, DEF_TEXT_SELECT_RELIEF);
     textPtr->selTagPtr->relief = TK_RELIEF_RAISED;
     textPtr->currentMarkPtr = TkTextSetMark(textPtr, "current", &startIndex);
     textPtr->insertMarkPtr = TkTextSetMark(textPtr, "insert", &startIndex);
 
-    TkClassOption(textPtr->tkwin, "Text",&argc,&argv);
+    TkClassOption(textPtr->tkwin, "Text", &argc, &argv);
     TkSetClassProcs(textPtr->tkwin, &textClass, (ClientData) textPtr);
     Tk_CreateEventHandler(textPtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
@@ -482,7 +480,7 @@ Tk_TextCmd(clientData, interp, argc, argv)
 	Tk_DestroyWindow(textPtr->tkwin);
 	return TCL_ERROR;
     }
-    interp->result = Tk_PathName(textPtr->tkwin);
+    Tcl_SetResult(interp, Tk_PathName(textPtr->tkwin), TCL_STATIC);
 
     return TCL_OK;
 }
@@ -540,7 +538,10 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    goto done;
 	}
 	if (TkTextCharBbox(textPtr, &index1, &x, &y, &width, &height) == 0) {
-	    sprintf(interp->result, "%d %d %d %d", x, y, width, height);
+	    char buf[TCL_INTEGER_SPACE * 4];
+	    
+	    sprintf(buf, "%d %d %d %d", x, y, width, height);
+	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	}
     } else if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
 	    && (length >= 2)) {
@@ -598,7 +599,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	} else {
 	    goto compareError;
 	}
-	interp->result = (value) ? "1" : "0";
+	Tcl_SetResult(interp, ((value) ? "1" : "0"), TCL_STATIC);
     } else if ((c == 'c') && (strncmp(argv[1], "configure", length) == 0)
 	    && (length >= 3)) {
 	if (argc == 2) {
@@ -620,7 +621,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    goto done;
 	}
 	if (argc == 2) {
-	    interp->result = (tkBTreeDebug) ? "1" : "0";
+	    Tcl_SetResult(interp, ((tkBTreeDebug) ? "1" : "0"), TCL_STATIC);
 	} else {
 	    if (Tcl_GetBoolean(interp, argv[2], &tkBTreeDebug) != TCL_OK) {
 		result = TCL_ERROR;
@@ -656,8 +657,10 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	}
 	if (TkTextDLineInfo(textPtr, &index1, &x, &y, &width, &height, &base)
 		== 0) {
-	    sprintf(interp->result, "%d %d %d %d %d", x, y, width,
-		    height, base);
+	    char buf[TCL_INTEGER_SPACE * 5];
+	    
+	    sprintf(buf, "%d %d %d %d %d", x, y, width, height, base);
+	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	}
     } else if ((c == 'g') && (strncmp(argv[1], "get", length) == 0)) {
 	if ((argc != 3) && (argc != 4)) {
@@ -690,10 +693,10 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    if (index1.linePtr == index2.linePtr) {
 		int last2;
 
-		if (index2.charIndex == index1.charIndex) {
+		if (index2.byteIndex == index1.byteIndex) {
 		    break;
 		}
-		last2 = index2.charIndex - index1.charIndex + offset;
+		last2 = index2.byteIndex - index1.byteIndex + offset;
 		if (last2 < last) {
 		    last = last2;
 		}
@@ -705,10 +708,12 @@ TextWidgetCmd(clientData, interp, argc, argv)
 			(char *) NULL);
 		segPtr->body.chars[last] = savedChar;
 	    }
-	    TkTextIndexForwChars(&index1, last-offset, &index1);
+	    TkTextIndexForwBytes(&index1, last-offset, &index1);
 	}
     } else if ((c == 'i') && (strncmp(argv[1], "index", length) == 0)
 	    && (length >= 3)) {
+	char buf[200];
+
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
 		    argv[0], " index index\"",
@@ -720,7 +725,8 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    result = TCL_ERROR;
 	    goto done;
 	}
-	TkTextPrintIndex(&index1, interp->result);
+	TkTextPrintIndex(&index1, buf);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
     } else if ((c == 'i') && (strncmp(argv[1], "insert", length) == 0)
 	    && (length >= 3)) {
 	int i, j, numTags;
@@ -743,7 +749,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    for (j = 3;  j < argc; j += 2) {
 		InsertChars(textPtr, &index1, argv[j]);
 		if (argc > (j+1)) {
-		    TkTextIndexForwChars(&index1, (int) strlen(argv[j]),
+		    TkTextIndexForwBytes(&index1, (int) strlen(argv[j]),
 			    &index2);
 		    oldTagArrayPtr = TkBTreeGetTags(&index1, &numTags);
 		    if (oldTagArrayPtr != NULL) {
@@ -913,7 +919,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message.
+ *	returned, then the interp's result contains an error message.
  *
  * Side effects:
  *	Configuration information, such as text string, colors, font,
@@ -1061,8 +1067,8 @@ ConfigureText(interp, textPtr, argc, argv, flags)
 	TkTextSearch search;
 	TkTextIndex first, last;
 
-	TkTextMakeIndex(textPtr->tree, 0, 0, &first);
-	TkTextMakeIndex(textPtr->tree,
+	TkTextMakeByteIndex(textPtr->tree, 0, 0, &first);
+	TkTextMakeByteIndex(textPtr->tree,
 		TkBTreeNumLines(textPtr->tree), 0, &last);
 	TkBTreeStartSearch(&first, &last, textPtr->selTagPtr, &search);
 	if (TkBTreeCharTagged(&first, textPtr->selTagPtr)
@@ -1293,7 +1299,7 @@ InsertChars(textPtr, indexPtr, string)
     lineIndex = TkBTreeLineIndex(indexPtr->linePtr);
     if (lineIndex == TkBTreeNumLines(textPtr->tree)) {
 	lineIndex--;
-	TkTextMakeIndex(textPtr->tree, lineIndex, 1000000, indexPtr);
+	TkTextMakeByteIndex(textPtr->tree, lineIndex, 1000000, indexPtr);
     }
 
     /*
@@ -1306,16 +1312,16 @@ InsertChars(textPtr, indexPtr, string)
     resetView = offset = 0;
     if (indexPtr->linePtr == textPtr->topIndex.linePtr) {
 	resetView = 1;
-	offset = textPtr->topIndex.charIndex;
-	if (offset > indexPtr->charIndex) {
+	offset = textPtr->topIndex.byteIndex;
+	if (offset > indexPtr->byteIndex) {
 	    offset += strlen(string);
 	}
     }
     TkTextChanged(textPtr, indexPtr, indexPtr);
     TkBTreeInsertChars(indexPtr, string);
     if (resetView) {
-	TkTextMakeIndex(textPtr->tree, lineIndex, 0, &newTop);
-	TkTextIndexForwChars(&newTop, offset, &newTop);
+	TkTextMakeByteIndex(textPtr->tree, lineIndex, 0, &newTop);
+	TkTextIndexForwBytes(&newTop, offset, &newTop);
 	TkTextSetYView(textPtr, &newTop, 0);
     }
 
@@ -1354,7 +1360,7 @@ DeleteChars(textPtr, index1String, index2String)
 				 * delete the one character given by
 				 * index1String. */
 {
-    int line1, line2, line, charIndex, resetView;
+    int line1, line2, line, byteIndex, resetView;
     TkTextIndex index1, index2;
 
     /*
@@ -1405,7 +1411,7 @@ DeleteChars(textPtr, index1String, index2String)
 	oldIndex2 = index2;
 	TkTextIndexBackChars(&oldIndex2, 1, &index2);
 	line2--;
-	if ((index1.charIndex == 0) && (line1 != 0)) {
+	if ((index1.byteIndex == 0) && (line1 != 0)) {
 	    TkTextIndexBackChars(&index1, 1, &index1);
 	    line1--;
 	}
@@ -1428,7 +1434,9 @@ DeleteChars(textPtr, index1String, index2String)
      */
 
     TkTextChanged(textPtr, &index1, &index2);
-    resetView = line = charIndex = 0;
+    resetView = 0;
+    line = 0;
+    byteIndex = 0;
     if (TkTextIndexCmp(&index2, &textPtr->topIndex) >= 0) {
 	if (TkTextIndexCmp(&index1, &textPtr->topIndex) <= 0) {
 	    /*
@@ -1438,7 +1446,7 @@ DeleteChars(textPtr, index1String, index2String)
 
 	    resetView = 1;
 	    line = line1;
-	    charIndex = index1.charIndex;
+	    byteIndex = index1.byteIndex;
 	} else if (index1.linePtr == textPtr->topIndex.linePtr) {
 	    /*
 	     * Deletion range starts on top line but after topIndex.
@@ -1447,7 +1455,7 @@ DeleteChars(textPtr, index1String, index2String)
 
 	    resetView = 1;
 	    line = line1;
-	    charIndex = textPtr->topIndex.charIndex;
+	    byteIndex = textPtr->topIndex.byteIndex;
 	}
     } else if (index2.linePtr == textPtr->topIndex.linePtr) {
 	/*
@@ -1458,16 +1466,16 @@ DeleteChars(textPtr, index1String, index2String)
 
 	resetView = 1;
 	line = line2;
-	charIndex = textPtr->topIndex.charIndex;
+	byteIndex = textPtr->topIndex.byteIndex;
 	if (index1.linePtr != index2.linePtr) {
-	    charIndex -= index2.charIndex;
+	    byteIndex -= index2.byteIndex;
 	} else {
-	    charIndex -= (index2.charIndex - index1.charIndex);
+	    byteIndex -= (index2.byteIndex - index1.byteIndex);
 	}
     }
     TkBTreeDeleteChars(&index1, &index2);
     if (resetView) {
-	TkTextMakeIndex(textPtr->tree, line, charIndex, &index1);
+	TkTextMakeByteIndex(textPtr->tree, line, byteIndex, &index1);
 	TkTextSetYView(textPtr, &index1, 0);
     }
 
@@ -1531,12 +1539,12 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
      */
 
     if (offset == 0) {
-	TkTextMakeIndex(textPtr->tree, 0, 0, &textPtr->selIndex);
+	TkTextMakeByteIndex(textPtr->tree, 0, 0, &textPtr->selIndex);
 	textPtr->abortSelections = 0;
     } else if (textPtr->abortSelections) {
 	return 0;
     }
-    TkTextMakeIndex(textPtr->tree, TkBTreeNumLines(textPtr->tree), 0, &eof);
+    TkTextMakeByteIndex(textPtr->tree, TkBTreeNumLines(textPtr->tree), 0, &eof);
     TkBTreeStartSearch(&textPtr->selIndex, &eof, textPtr->selTagPtr, &search);
     if (!TkBTreeCharTagged(&textPtr->selIndex, textPtr->selTagPtr)) {
 	if (!TkBTreeNextTag(&search)) {
@@ -1583,8 +1591,8 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
 	    if (textPtr->selIndex.linePtr == search.curIndex.linePtr) {
 		int leftInRange;
 
-		leftInRange = search.curIndex.charIndex
-			- textPtr->selIndex.charIndex;
+		leftInRange = search.curIndex.byteIndex
+			- textPtr->selIndex.byteIndex;
 		if (leftInRange < chunkSize) {
 		    chunkSize = leftInRange;
 		    if (chunkSize <= 0) {
@@ -1599,7 +1607,7 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
 		maxBytes -= chunkSize;
 		count += chunkSize;
 	    }
-	    TkTextIndexForwChars(&textPtr->selIndex, chunkSize,
+	    TkTextIndexForwBytes(&textPtr->selIndex, chunkSize,
 		    &textPtr->selIndex);
 	}
 
@@ -1656,8 +1664,8 @@ TkTextLostSelection(clientData)
      * just remove the "sel" tag from everything in the widget.
      */
 
-    TkTextMakeIndex(textPtr->tree, 0, 0, &start);
-    TkTextMakeIndex(textPtr->tree, TkBTreeNumLines(textPtr->tree), 0, &end);
+    TkTextMakeByteIndex(textPtr->tree, 0, 0, &start);
+    TkTextMakeByteIndex(textPtr->tree, TkBTreeNumLines(textPtr->tree), 0, &end);
     TkTextRedrawTag(textPtr, &start, &end, textPtr->selTagPtr, 1);
     TkBTreeTag(&start, &end, textPtr->selTagPtr, 0);
 #endif
@@ -1735,8 +1743,8 @@ TextSearchCmd(textPtr, interp, argc, argv)
 {
     int backwards, exact, searchElide, c, i, argsLeft, noCase, leftToScan;
     size_t length;
-    int numLines, startingLine, startingChar, lineNum, firstChar, lastChar;
-    int code, matchLength, matchChar, passes, stopLine, searchWholeText;
+    int numLines, startingLine, startingByte, lineNum, firstByte, lastByte;
+    int code, matchLength, matchByte, passes, stopLine, searchWholeText;
     int patLength;
     char *arg, *pattern, *varName, *p, *startOfLine;
     char buffer[20];
@@ -1776,7 +1784,8 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	    backwards = 1;
 	} else if ((c == 'c') && (strncmp(argv[i], "-count", length) == 0)) {
 	    if (i >= (argc-1)) {
-		interp->result = "no value given for \"-count\" option";
+		Tcl_SetResult(interp, "no value given for \"-count\" option",
+			TCL_STATIC);
 		return TCL_ERROR;
 	    }
 	    i++;
@@ -1817,32 +1826,31 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	Tcl_DStringInit(&patDString);
 	Tcl_DStringAppend(&patDString, pattern, -1);
 	pattern = Tcl_DStringValue(&patDString);
-	for (p = pattern; *p != 0; p++) {
-	    if (isupper(UCHAR(*p))) {
-		*p = tolower(UCHAR(*p));
-	    }
-	}
+	Tcl_UtfToLower(pattern);
     }
 
+    Tcl_DStringInit(&line);
     if (TkTextGetIndex(interp, textPtr, argv[i+1], &index) != TCL_OK) {
-	return TCL_ERROR;
+	code = TCL_ERROR;
+	goto done;
     }
     numLines = TkBTreeNumLines(textPtr->tree);
     startingLine = TkBTreeLineIndex(index.linePtr);
-    startingChar = index.charIndex;
+    startingByte = index.byteIndex;
     if (startingLine >= numLines) {
 	if (backwards) {
 	    startingLine = TkBTreeNumLines(textPtr->tree) - 1;
-	    startingChar = TkBTreeCharsInLine(TkBTreeFindLine(textPtr->tree,
+	    startingByte = TkBTreeBytesInLine(TkBTreeFindLine(textPtr->tree,
 		    startingLine));
 	} else {
 	    startingLine = 0;
-	    startingChar = 0;
+	    startingByte = 0;
 	}
     }
     if (argsLeft == 1) {
 	if (TkTextGetIndex(interp, textPtr, argv[i+2], &stopIndex) != TCL_OK) {
-	    return TCL_ERROR;
+	    code = TCL_ERROR;
+	    goto done;
 	}
 	stopLine = TkBTreeLineIndex(stopIndex.linePtr);
 	if (!backwards && (stopLine == numLines)) {
@@ -1866,12 +1874,12 @@ TextSearchCmd(textPtr, interp, argc, argv)
     } else {
 	regexp = Tcl_RegExpCompile(interp, pattern);
 	if (regexp == NULL) {
-	    return TCL_ERROR;
+	    code = TCL_ERROR;
+	    goto done;
 	}
     }
     lineNum = startingLine;
     code = TCL_OK;
-    Tcl_DStringInit(&line);
     for (passes = 0; passes < 2; ) {
 	if (lineNum >= numLines) {
 	    /*
@@ -1888,9 +1896,9 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	 */
 
 	linePtr = TkBTreeFindLine(textPtr->tree, lineNum);
-	curIndex.linePtr = linePtr; curIndex.charIndex = 0;
+	curIndex.linePtr = linePtr; curIndex.byteIndex = 0;
 	for (segPtr = linePtr->segPtr; segPtr != NULL;
-		curIndex.charIndex += segPtr->size, segPtr = segPtr->nextPtr) {
+		curIndex.byteIndex += segPtr->size, segPtr = segPtr->nextPtr) {
 	    if (segPtr->typePtr != &tkTextCharType || (!searchElide && TkTextIsElided(textPtr, &curIndex))) {
 		continue;
 	    }
@@ -1906,22 +1914,20 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	 */
 
 	if (noCase) {
-	    for (p = Tcl_DStringValue(&line); *p != 0; p++) {
-		if (isupper(UCHAR(*p))) {
-		    *p = tolower(UCHAR(*p));
-		}
-	    }
+	    Tcl_DStringSetLength(&line,
+		    Tcl_UtfToLower(Tcl_DStringValue(&line)));
 	}
 
 	/*
 	 * Check for matches within the current line.  If so, and if we're
 	 * searching backwards, repeat the search to find the last match
-	 * in the line.
+	 * in the line.  (Note: The lastByte should include the NULL char
+	 * so we can handle searching for end of line easier.)
 	 */
 
-	matchChar = -1;
-	firstChar = 0;
-	lastChar = INT_MAX;
+	matchByte = -1;
+	firstByte = 0;
+	lastByte = Tcl_DStringLength(&line) + 1;
 	if (lineNum == startingLine) {
 	    int indexInDString;
 
@@ -1935,8 +1941,8 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	     * character.
 	     */
 
-	    indexInDString = startingChar;
-	    for (segPtr = linePtr->segPtr, leftToScan = startingChar;
+	    indexInDString = startingByte;
+	    for (segPtr = linePtr->segPtr, leftToScan = startingByte;
 		    leftToScan > 0; segPtr = segPtr->nextPtr) {
 		if (segPtr->typePtr != &tkTextCharType) {
 		    indexInDString -= segPtr->size;
@@ -1950,8 +1956,8 @@ TextSearchCmd(textPtr, interp, argc, argv)
 		 * Only use the last part of the line.
 		 */
 
-		firstChar = indexInDString;
-		if (firstChar >= Tcl_DStringLength(&line)) {
+		firstByte = indexInDString;
+		if (firstByte >= Tcl_DStringLength(&line)) {
 		    goto nextLine;
 		}
 	    } else {
@@ -1959,13 +1965,16 @@ TextSearchCmd(textPtr, interp, argc, argv)
 		 * Use only the first part of the line.
 		 */
 
-		lastChar = indexInDString;
+		lastByte = indexInDString;
 	    }
 	}
 	do {
 	    int thisLength;
+	    Tcl_UniChar ch;
+
 	    if (exact) {
-		p = strstr(startOfLine + firstChar, pattern);
+		p = strstr(startOfLine + firstByte,	/* INTL: Native. */
+			pattern); 
 		if (p == NULL) {
 		    break;
 		}
@@ -1976,7 +1985,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
 		int match;
 
 		match = Tcl_RegExpExec(interp, regexp,
-			startOfLine + firstChar, startOfLine);
+			startOfLine + firstByte, startOfLine);
 		if (match < 0) {
 		    code = TCL_ERROR;
 		    goto done;
@@ -1988,12 +1997,12 @@ TextSearchCmd(textPtr, interp, argc, argv)
 		i = start - startOfLine;
 		thisLength = end - start;
 	    }
-	    if (i >= lastChar) {
+	    if (i >= lastByte) {
 		break;
 	    }
-	    matchChar = i;
+	    matchByte = i;
 	    matchLength = thisLength;
-	    firstChar = matchChar+1;
+	    firstByte += Tcl_UtfToUniChar(startOfLine + matchByte, &ch);
 	} while (backwards);
 
 	/*
@@ -2002,7 +2011,16 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	 * specified.
 	 */
 
-	if (matchChar >= 0) {
+	if (matchByte >= 0) {
+	    int numChars;
+
+	    /*
+	     * Convert the byte length to a character count.
+	     */
+
+	    numChars = Tcl_NumUtfChars(startOfLine + matchByte,
+		    matchLength);
+
 	    /*
 	     * The index information returned by the regular expression
 	     * parser only considers textual information:  it doesn't
@@ -2011,10 +2029,10 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	     * matchChar and matchCount.
 	     */
 
-	    for (segPtr = linePtr->segPtr, leftToScan = matchChar;
+	    for (segPtr = linePtr->segPtr, leftToScan = matchByte;
 		    leftToScan >= 0; segPtr = segPtr->nextPtr) {
 		if (segPtr->typePtr != &tkTextCharType) {
-		    matchChar += segPtr->size;
+		    matchByte += segPtr->size;
 		    continue;
 		}
 		leftToScan -= segPtr->size;
@@ -2022,12 +2040,12 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	    for (leftToScan += matchLength; leftToScan > 0;
 		    segPtr = segPtr->nextPtr) {
 		if (segPtr->typePtr != &tkTextCharType) {
-		    matchLength += segPtr->size;
+		    numChars += segPtr->size;
 		    continue;
 		}
 		leftToScan -= segPtr->size;
 	    }
-	    TkTextMakeIndex(textPtr->tree, lineNum, matchChar, &index);
+	    TkTextMakeByteIndex(textPtr->tree, lineNum, matchByte, &index);
 	    if (!searchWholeText) {
 		if (!backwards && (TkTextIndexCmp(&index, &stopIndex) >= 0)) {
 		    goto done;
@@ -2037,14 +2055,15 @@ TextSearchCmd(textPtr, interp, argc, argv)
 		}
 	    }
 	    if (varName != NULL) {
-		sprintf(buffer, "%d", matchLength);
+		sprintf(buffer, "%d", numChars);
 		if (Tcl_SetVar(interp, varName, buffer, TCL_LEAVE_ERR_MSG)
 			== NULL) {
 		    code = TCL_ERROR;
 		    goto done;
 		}
 	    }
-	    TkTextPrintIndex(&index, interp->result);
+	    TkTextPrintIndex(&index, buffer);
+	    Tcl_SetResult(interp, buffer, TCL_VOLATILE);
 	    goto done;
 	}
 
@@ -2093,7 +2112,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
  *	The return value is a pointer to a malloc'ed structure holding
  *	parsed information about the tab stops.  If an error occurred
  *	then the return value is NULL and an error message is left in
- *	interp->result.
+ *	the interp's result.
  *
  * Side effects:
  *	Memory is allocated for the structure that is returned.  It is
@@ -2115,6 +2134,7 @@ TkTextGetTabs(interp, tkwin, string)
     char **argv;
     TkTextTabArray *tabArrayPtr;
     TkTextTab *tabPtr;
+    Tcl_UniChar ch;
 
     if (Tcl_SplitList(interp, string, &argc, &argv) != TCL_OK) {
 	return NULL;
@@ -2157,11 +2177,12 @@ TkTextGetTabs(interp, tkwin, string)
 	if ((i+1) == argc) {
 	    continue;
 	}
-	c = UCHAR(argv[i+1][0]);
-	if (!isalpha(c)) {
+	Tcl_UtfToUniChar(argv[i+1], &ch);
+	if (!Tcl_UniCharIsAlpha(ch)) {
 	    continue;
 	}
 	i += 1;
+	c = argv[i][0];
 	if ((c == 'l') && (strncmp(argv[i], "left",
 		strlen(argv[i])) == 0)) {
 	    tabPtr->alignment = LEFT;
@@ -2291,10 +2312,10 @@ TextDumpCmd(textPtr, interp, argc, argv)
     }
     if (index1.linePtr == index2.linePtr) {
 	DumpLine(interp, textPtr, what, index1.linePtr,
-	    index1.charIndex, index2.charIndex, lineno, command);
+	    index1.byteIndex, index2.byteIndex, lineno, command);
     } else {
 	DumpLine(interp, textPtr, what, index1.linePtr,
-		index1.charIndex, 32000000, lineno, command);
+		index1.byteIndex, 32000000, lineno, command);
 	linePtr = index1.linePtr;
 	while ((linePtr = TkBTreeNextLine(linePtr)) != (TkTextLine *)NULL) {
 	    lineno++;
@@ -2305,14 +2326,14 @@ TextDumpCmd(textPtr, interp, argc, argv)
 		    lineno, command);
 	}
 	DumpLine(interp, textPtr, what, index2.linePtr, 0,
-		index2.charIndex, lineno, command);
+		index2.byteIndex, lineno, command);
     }
     /*
      * Special case to get the leftovers hiding at the end mark.
      */
     if (atEnd) {
 	DumpLine(interp, textPtr, what & ~TK_DUMP_TEXT, index2.linePtr,
-		0, 1, lineno, command);
+		0, 1, lineno, command);			    
 
     }
     return TCL_OK;
@@ -2330,12 +2351,12 @@ TextDumpCmd(textPtr, interp, argc, argv)
  *	None, but see DumpSegment.
  */
 static void
-DumpLine(interp, textPtr, what, linePtr, start, end, lineno, command)
+DumpLine(interp, textPtr, what, linePtr, startByte, endByte, lineno, command)
     Tcl_Interp *interp;
     TkText *textPtr;
     int what;			/* bit flags to select segment types */
     TkTextLine *linePtr;	/* The current line */
-    int start, end;		/* Character range to dump */
+    int startByte, endByte;	/* Byte range to dump */
     int lineno;			/* Line number for indices dump */
     char *command;		/* Script to apply to the segment */
 {
@@ -2350,25 +2371,25 @@ DumpLine(interp, textPtr, what, linePtr, start, end, lineno, command)
      * window
      */
     for (offset = 0, segPtr = linePtr->segPtr ;
-	    (offset < end) && (segPtr != (TkTextSegment *)NULL) ;
+	    (offset < endByte) && (segPtr != (TkTextSegment *)NULL) ;
 	    offset += segPtr->size, segPtr = segPtr->nextPtr) {
 	if ((what & TK_DUMP_TEXT) && (segPtr->typePtr == &tkTextCharType) &&
-		(offset + segPtr->size > start)) {
+		(offset + segPtr->size > startByte)) {
 	    char savedChar;			/* Last char used in the seg */
 	    int last = segPtr->size;		/* Index of savedChar */
 	    int first = 0;			/* Index of first char in seg */
-	    if (offset + segPtr->size > end) {
-		last = end - offset;
+	    if (offset + segPtr->size > endByte) {
+		last = endByte - offset;
 	    }
-	    if (start > offset) {
-		first = start - offset;
+	    if (startByte > offset) {
+		first = startByte - offset;
 	    }
 	    savedChar = segPtr->body.chars[last];
 	    segPtr->body.chars[last] = '\0';
 	    DumpSegment(interp, "text", segPtr->body.chars + first,
 		    command, lineno, offset + first, what);
 	    segPtr->body.chars[last] = savedChar;
-	} else if ((offset >= start)) {
+	} else if ((offset >= startByte)) {
 	    if ((what & TK_DUMP_MARK) && (segPtr->typePtr->name[0] == 'm')) {
 		TkTextMark *markPtr = (TkTextMark *)&segPtr->body;
 		char *name = Tcl_GetHashKey(&textPtr->markTable, markPtr->hPtr);
@@ -2424,11 +2445,11 @@ DumpSegment(interp, key, value, command, lineno, offset, what)
     char *value;		/* Segment value */
     char *command;		/* Script callback */
     int lineno;			/* Line number for indices dump */
-    int offset;			/* Character position */
+    int offset;			/* Byte position */
     int what;			/* Look for TK_DUMP_INDEX bit */
 {
     char buffer[30];
-    sprintf(buffer, "%d.%d", lineno, offset);
+    sprintf(buffer, "%d.%d", lineno, offset);		
     if (command == (char *) NULL) {
 	Tcl_AppendElement(interp, key);
 	Tcl_AppendElement(interp, value);
