@@ -6,12 +6,12 @@
  *	a focus highlight.
  *
  * Copyright (c) 1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkUtil.c 1.13 97/06/06 11:16:22
+ * RCS: @(#) $Id: tkUtil.c,v 1.5 1999/04/21 21:53:28 rjohnson Exp $
  */
 
 #include "tkInt.h"
@@ -34,7 +34,7 @@ static Tcl_ObjType stateKeyType = {
 /*
  *--------------------------------------------------------------
  *
- * Tk_StateParseProc --
+ * TkStateParseProc --
  *
  *	This procedure is invoked during option processing to handle
  *	the "-state" and "-default" options.
@@ -50,7 +50,7 @@ static Tcl_ObjType stateKeyType = {
  */
 
 int
-Tk_StateParseProc(clientData, interp, tkwin, value, widgRec, offset)
+TkStateParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;		/* some flags.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
@@ -109,7 +109,7 @@ Tk_StateParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *--------------------------------------------------------------
  *
- * Tk_StatePrintProc --
+ * TkStatePrintProc --
  *
  *	This procedure is invoked by the Tk configuration code
  *	to produce a printable string for the "-state"
@@ -129,7 +129,7 @@ Tk_StateParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 
 char *
-Tk_StatePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+TkStatePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
     char *widgRec;			/* Pointer to record for item. */
@@ -156,7 +156,7 @@ Tk_StatePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 /*
  *--------------------------------------------------------------
  *
- * Tk_OrientParseProc --
+ * TkOrientParseProc --
  *
  *	This procedure is invoked during option processing to handle
  *	the "-orient" option.
@@ -172,7 +172,7 @@ Tk_StatePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  */
 
 int
-Tk_OrientParseProc(clientData, interp, tkwin, value, widgRec, offset)
+TkOrientParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;		/* some flags.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
@@ -211,7 +211,7 @@ Tk_OrientParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *--------------------------------------------------------------
  *
- * Tk_OrientPrintProc --
+ * TkOrientPrintProc --
  *
  *	This procedure is invoked by the Tk configuration code
  *	to produce a printable string for the "-orient"
@@ -231,7 +231,7 @@ Tk_OrientParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 
 char *
-Tk_OrientPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+TkOrientPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
     char *widgRec;			/* Pointer to record for item. */
@@ -252,7 +252,7 @@ Tk_OrientPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_TileParseProc --
+ * TkTileParseProc --
  *
  *	Converts the name of an image into a tile.
  *
@@ -260,7 +260,7 @@ Tk_OrientPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  */
 
 int
-Tk_TileParseProc(clientData, interp, tkwin, value, widgRec, offset)
+TkTileParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window on same display as tile */
@@ -289,7 +289,7 @@ Tk_TileParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_TilePrintProc --
+ * TkTilePrintProc --
  *
  *	Returns the name of the tile.
  *
@@ -300,7 +300,7 @@ Tk_TileParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 
 char *
-Tk_TilePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+TkTilePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Widget structure record */
@@ -315,7 +315,7 @@ Tk_TilePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_OffsetParseProc --
+ * TkOffsetParseProc --
  *
  *	Converts the offset of a stipple or tile into the Tk_TSOffset structure.
  *
@@ -323,7 +323,7 @@ Tk_TilePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  */
 
 int
-Tk_OffsetParseProc(clientData, interp, tkwin, value, widgRec, offset)
+TkOffsetParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window on same display as tile */
@@ -447,7 +447,7 @@ badTSOffset:
 /*
  *----------------------------------------------------------------------
  *
- * Tk_OffsetPrintProc --
+ * TkOffsetPrintProc --
  *
  *	Returns the offset of the tile.
  *
@@ -458,7 +458,7 @@ badTSOffset:
  */
 
 char *
-Tk_OffsetPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+TkOffsetPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Widget structure record */
@@ -515,7 +515,7 @@ Tk_OffsetPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_PixelParseProc --
+ * TkPixelParseProc --
  *
  *	Converts the name of an image into a tile.
  *
@@ -523,7 +523,7 @@ Tk_OffsetPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
  */
 
 int
-Tk_PixelParseProc(clientData, interp, tkwin, value, widgRec, offset)
+TkPixelParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;	/* if non-NULL, negative values are
 				 * allowed as well */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
@@ -535,7 +535,7 @@ Tk_PixelParseProc(clientData, interp, tkwin, value, widgRec, offset)
     double *doublePtr = (double *)(widgRec + offset);
     int result;
 
-    result = Tk_GetDoublePixels(interp, tkwin, value, doublePtr);
+    result = TkGetDoublePixels(interp, tkwin, value, doublePtr);
 
     if ((result == TCL_OK) && (clientData == NULL) && (*doublePtr < 0.0)) {
 	Tcl_AppendResult(interp, "bad screen distance \"", value,
@@ -548,7 +548,7 @@ Tk_PixelParseProc(clientData, interp, tkwin, value, widgRec, offset)
 /*
  *----------------------------------------------------------------------
  *
- * Tk_PixelPrintProc --
+ * TkPixelPrintProc --
  *
  *	Returns the name of the tile.
  *
@@ -559,7 +559,7 @@ Tk_PixelParseProc(clientData, interp, tkwin, value, widgRec, offset)
  */
 
 char *
-Tk_PixelPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+TkPixelPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
     char *widgRec;		/* Widget structure record */
@@ -616,7 +616,7 @@ TkDrawInsetFocusHighlight(tkwin, gc, width, drawable, padding)
      * two things to worry about:
      *
      * 1) The widget must draw the background color covering
-     *    the focus ring area before calling Tk_DrawFocus.
+     *    the focus ring area before calling Tk_DrawFocusHighlight.
      * 2) It is impossible to draw a focus ring of width 1.
      *    (For the Macintosh Look & Feel use width of 3)
      */
@@ -690,7 +690,7 @@ Tk_DrawFocusHighlight(tkwin, gc, width, drawable)
  *	took.  If TK_SCROLL_MOVETO, *dblPtr is filled in with the
  *	desired position;  if TK_SCROLL_PAGES or TK_SCROLL_UNITS,
  *	*intPtr is filled in with the number of lines to move (may be
- *	negative);  if TK_SCROLL_ERROR, interp->result contains an
+ *	negative);  if TK_SCROLL_ERROR, the interp's result contains an
  *	error message.
  *
  * Side effects:
@@ -754,6 +754,85 @@ Tk_GetScrollInfo(interp, argc, argv, dblPtr, intPtr)
     return TK_SCROLL_ERROR;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tk_GetScrollInfoObj --
+ *
+ *	This procedure is invoked to parse "xview" and "yview"
+ *	scrolling commands for widgets using the new scrolling
+ *	command syntax ("moveto" or "scroll" options).
+ *
+ * Results:
+ *	The return value is either TK_SCROLL_MOVETO, TK_SCROLL_PAGES,
+ *	TK_SCROLL_UNITS, or TK_SCROLL_ERROR.  This indicates whether
+ *	the command was successfully parsed and what form the command
+ *	took.  If TK_SCROLL_MOVETO, *dblPtr is filled in with the
+ *	desired position;  if TK_SCROLL_PAGES or TK_SCROLL_UNITS,
+ *	*intPtr is filled in with the number of lines to move (may be
+ *	negative);  if TK_SCROLL_ERROR, the interp's result contains an
+ *	error message.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Tk_GetScrollInfoObj(interp, objc, objv, dblPtr, intPtr)
+    Tcl_Interp *interp;			/* Used for error reporting. */
+    int objc;				/* # arguments for command. */
+    Tcl_Obj *CONST objv[];		/* Arguments for command. */
+    double *dblPtr;			/* Filled in with argument "moveto"
+					 * option, if any. */
+    int *intPtr;			/* Filled in with number of pages
+					 * or lines to scroll, if any. */
+{
+    int c;
+    size_t length;
+    char *arg2, *arg4;
+
+    arg2 = Tcl_GetString(objv[2]);
+    length = strlen(arg2);
+    c = arg2[0];
+    if ((c == 'm') && (strncmp(arg2, "moveto", length) == 0)) {
+	if (objc != 4) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "moveto fraction");
+	    return TK_SCROLL_ERROR;
+	}
+	if (Tcl_GetDoubleFromObj(interp, objv[3], dblPtr) != TCL_OK) {
+	    return TK_SCROLL_ERROR;
+	}
+	return TK_SCROLL_MOVETO;
+    } else if ((c == 's')
+	    && (strncmp(arg2, "scroll", length) == 0)) {
+	if (objc != 5) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "scroll number units|pages");
+	    return TK_SCROLL_ERROR;
+	}
+	if (Tcl_GetIntFromObj(interp, objv[3], intPtr) != TCL_OK) {
+	    return TK_SCROLL_ERROR;
+	}
+	arg4 = Tcl_GetString(objv[4]);
+	length = (strlen(arg4));
+	c = arg4[0];
+	if ((c == 'p') && (strncmp(arg4, "pages", length) == 0)) {
+	    return TK_SCROLL_PAGES;
+	} else if ((c == 'u')
+		&& (strncmp(arg4, "units", length) == 0)) {
+	    return TK_SCROLL_UNITS;
+	} else {
+	    Tcl_AppendResult(interp, "bad argument \"", arg4,
+		    "\": must be units or pages", (char *) NULL);
+	    return TK_SCROLL_ERROR;
+	}
+    }
+    Tcl_AppendResult(interp, "unknown option \"", arg2,
+	    "\": must be moveto or scroll", (char *) NULL);
+    return TK_SCROLL_ERROR;
+}
+
 /*
  *---------------------------------------------------------------------------
  *
@@ -868,7 +947,7 @@ TkFindStateString(mapPtr, numKey)
  *	Returns the numKey associated with the last element (the NULL
  *	string one) in the table if strKey was not equal to any of the
  *	string keys in the table.  In that case, an error message is
- *	also left in interp->result (if interp is not NULL).
+ *	also left in the interp's result (if interp is not NULL).
  *
  * Side effects.
  *	None.
@@ -877,18 +956,14 @@ TkFindStateString(mapPtr, numKey)
  */
 
 int
-TkFindStateNum(interp, field, mapPtr, strKey)
+TkFindStateNum(interp, option, mapPtr, strKey)
     Tcl_Interp *interp;		/* Interp for error reporting. */
-    CONST char *field;		/* String to use when constructing error. */
+    CONST char *option;		/* String to use when constructing error. */
     CONST TkStateMap *mapPtr;	/* Lookup table. */
     CONST char *strKey;		/* String to try to find in lookup table. */
 {
     CONST TkStateMap *mPtr;
 
-    if (mapPtr->strKey == NULL) {
-	panic("TkFindStateNum: no choices in lookup table");
-    }
-
     for (mPtr = mapPtr; mPtr->strKey != NULL; mPtr++) {
 	if (strcmp(strKey, mPtr->strKey) == 0) {
 	    return mPtr->numKey;
@@ -896,14 +971,17 @@ TkFindStateNum(interp, field, mapPtr, strKey)
     }
     if (interp != NULL) {
 	mPtr = mapPtr;
-	Tcl_AppendResult(interp, "bad ", field, " value \"", strKey,
+	Tcl_AppendResult(interp, "bad ", option, " value \"", strKey,
 		"\": must be ", mPtr->strKey, (char *) NULL);
 	for (mPtr++; mPtr->strKey != NULL; mPtr++) {
-	    Tcl_AppendResult(interp, ", ", mPtr->strKey, (char *) NULL);
+	    Tcl_AppendResult(interp, 
+		    ((mPtr[1].strKey != NULL) ? ", " : ", or "), 
+		    mPtr->strKey, (char *) NULL);
 	}
     }
     return mPtr->numKey;
 }
+
 int
 TkFindStateNumObj(interp, optionPtr, mapPtr, keyPtr)
     Tcl_Interp *interp;		/* Interp for error reporting. */
@@ -939,7 +1017,9 @@ TkFindStateNumObj(interp, optionPtr, mapPtr, keyPtr)
 		Tcl_GetStringFromObj(optionPtr, NULL), " value \"", key,
 		"\": must be ", mPtr->strKey, (char *) NULL);
 	for (mPtr++; mPtr->strKey != NULL; mPtr++) {
-	    Tcl_AppendResult(interp, ", ", mPtr->strKey, (char *) NULL);
+	    Tcl_AppendResult(interp, 
+		((mPtr[1].strKey != NULL) ? ", " : ", or "), 
+		mPtr->strKey, (char *) NULL);
 	}
     }
     return mPtr->numKey;
