@@ -3,10 +3,11 @@
 # This file defines the default bindings for Tk text widgets and provides
 # procedures that help in implementing the bindings.
 #
-# SCCS: @(#) text.tcl 1.58 97/09/17 18:54:56
+# RCS: @(#) $Id: text.tcl,v 1.6 1999/04/16 01:51:27 stanton Exp $
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1997 Sun Microsystems, Inc.
+# Copyright (c) 1998 by Scriptics Corporation.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -59,6 +60,11 @@ bind Text <Triple-1> {
     tkTextSelectTo %W %x %y
     catch {%W mark set insert sel.first}
 }
+bind Text <Quadruple-1> {
+    set tkPriv(selectMode) all
+    tkTextSelectTo %W %x %y
+    catch {%W mark set insert sel.first}
+}
 bind Text <Shift-1> {
     tkTextResetAnchor %W @%x,%y
     set tkPriv(selectMode) char
@@ -72,6 +78,10 @@ bind Text <Triple-Shift-1>	{
     set tkPriv(selectMode) line
     tkTextSelectTo %W %x %y
 }
+bind Text <Quadruple-Shift-1> {
+    set tkPriv(selectMode) all
+    tkTextSelectTo %W %x %y
+}
 bind Text <B1-Leave> {
     set tkPriv(x) %x
     set tkPriv(y) %y
@@ -86,11 +96,6 @@ bind Text <ButtonRelease-1> {
 bind Text <Control-1> {
     %W mark set insert @%x,%y
 }
-bind Text <ButtonRelease-2> {
-    if {!$tkPriv(mouseMoved) || $tk_strictMotif} {
-	tkTextPaste %W %x %y
-    }
-}
 bind Text <Left> {
     tkTextSetCursor %W insert-1c
 }
@@ -206,7 +211,7 @@ bind Text <Return> {
     tkTextInsert %W \n
 }
 bind Text <Delete> {
-    if {[%W tag nextrange sel 1.0 end] != ""} {
+    if {[string compare [%W tag nextrange sel 1.0 end] ""]} {
 	%W delete sel.first sel.last
     } else {
 	%W delete insert
@@ -214,7 +219,7 @@ bind Text <Delete> {
     }
 }
 bind Text <BackSpace> {
-    if {[%W tag nextrange sel 1.0 end] != ""} {
+    if {[string compare [%W tag nextrange sel 1.0 end] ""]} {
 	%W delete sel.first sel.last
     } elseif {[%W compare insert != 1.0]} {
 	%W delete insert-1c
@@ -254,6 +259,11 @@ bind Text <<Paste>> {
 bind Text <<Clear>> {
     catch {%W delete sel.first sel.last}
 }
+bind Text <<PasteSelection>> {
+    if {!$tkPriv(mouseMoved) || $tk_strictMotif} {
+	tkTextPaste %W %x %y
+    }
+}
 bind Text <Insert> {
     catch {tkTextInsert %W [selection get -displayof %W]}
 }
@@ -271,7 +281,7 @@ bind Text <Meta-KeyPress> {# nothing}
 bind Text <Control-KeyPress> {# nothing}
 bind Text <Escape> {# nothing}
 bind Text <KP_Enter> {# nothing}
-if {$tcl_platform(platform) == "macintosh"} {
+if {![string compare $tcl_platform(platform) "macintosh"]} {
 	bind Text <Command-KeyPress> {# nothing}
 }
 
@@ -333,7 +343,7 @@ bind Text <Control-t> {
     }
 }
 
-if {$tcl_platform(platform) != "windows"} {
+if {[string compare $tcl_platform(platform) "windows"]} {
 bind Text <Control-v> {
     if {!$tk_strictMotif} {
 	tkTextScrollPages %W 1
@@ -380,7 +390,7 @@ bind Text <Meta-Delete> {
 # Macintosh only bindings:
 
 # if text black & highlight black -> text white, other text the same
-if {$tcl_platform(platform) == "macintosh"} {
+if {![string compare $tcl_platform(platform) "macintosh"]} {
 bind Text <FocusIn> {
     %W tag configure sel -borderwidth 0
     %W configure -selectbackground systemHighlight -selectforeground systemHighlightText
@@ -447,6 +457,14 @@ bind Text <B2-Motion> {
 }
 set tkPriv(prevPos) {}
 
+# The MouseWheel will typically only fire on Windows.  However,
+# someone could use the "event generate" command to produce one
+# on other platforms.
+
+bind Text <MouseWheel> {
+    %W yview scroll [expr {- (%D / 120) * 4}] units
+}
+
 # tkTextClosestGap --
 # Given x and y coordinates, this procedure finds the closest boundary
 # between characters to the given coordinates and returns the index
@@ -487,13 +505,13 @@ proc tkTextButton1 {w x y} {
     set tkPriv(pressX) $x
     $w mark set insert [tkTextClosestGap $w $x $y]
     $w mark set anchor insert
-    if {[$w cget -state] == "normal"} {focus $w}
+    if {![string compare [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkTextSelectTo --
 # This procedure is invoked to extend the selection, typically when
 # dragging it with the mouse.  Depending on the selection mode (character,
-# word, line) it selects in different-sized units.  This procedure
+# word, line, all) it selects in different-sized units.  This procedure
 # ignores mouse motions initially until the mouse has moved from
 # one character to another or until there have been multiple clicks.
 #
@@ -541,9 +559,19 @@ proc tkTextSelectTo {w x y} {
 		set last [$w index "$cur lineend + 1c"]
 	    }
 	}
+	all {
+	    if {[$w compare $cur < anchor]} {
+		set first [$w index "$cur linestart"]
+		set last [$w index end]
+	    } else {
+		set first [$w index 0,0]
+		set last [$w index "$cur lineend + 1c"]
+	    }
+	}
     }
-    if {$tkPriv(mouseMoved) || ($tkPriv(selectMode) != "char")} {
-	if {$tcl_platform(platform) != "unix" && [$w compare $cur < anchor]} {
+    if {$tkPriv(mouseMoved) || [string compare $tkPriv(selectMode) "char"]} {
+      if {[string compare $tcl_platform(platform) "unix"]
+              && [$w compare $cur < anchor]} {
 	    $w mark set insert $first
 	} else {
 	    $w mark set insert $last
@@ -595,7 +623,7 @@ proc tkTextKeyExtend {w index} {
 proc tkTextPaste {w x y} {
     $w mark set insert [tkTextClosestGap $w $x $y]
     catch {$w insert insert [selection get -displayof $w]}
-    if {[$w cget -state] == "normal"} {focus $w}
+    if {![string compare [$w cget -state] "normal"]} {focus $w}
 }
 
 # tkTextAutoScan --
@@ -661,7 +689,7 @@ proc tkTextSetCursor {w pos} {
 proc tkTextKeySelect {w new} {
     global tkPriv
 
-    if {[$w tag nextrange sel 1.0 end] == ""} {
+    if {![string compare [$w tag nextrange sel 1.0 end] ""]} {
 	if {[$w compare $new < insert]} {
 	    $w tag add sel $new insert
 	} else {
@@ -702,7 +730,7 @@ proc tkTextKeySelect {w new} {
 proc tkTextResetAnchor {w index} {
     global tkPriv
 
-    if {[$w tag ranges sel] == ""} {
+    if {![string compare [$w tag ranges sel] ""]} {
 	$w mark set anchor $index
 	return
     }
@@ -749,7 +777,8 @@ proc tkTextResetAnchor {w index} {
 # s -		The string to insert (usually just a single character)
 
 proc tkTextInsert {w s} {
-    if {($s == "") || ([$w cget -state] == "disabled")} {
+    if {![string compare $s ""] ||
+          ![string compare [$w cget -state] "disabled"]} {
 	return
     }
     catch {
@@ -780,11 +809,11 @@ proc tkTextUpDownLine {w n} {
 
     set i [$w index insert]
     scan $i "%d.%d" line char
-    if {[string compare $tkPriv(prevPos) $i] != 0} {
+    if {[string compare $tkPriv(prevPos) $i]} {
 	set tkPriv(char) $char
     }
     set new [$w index [expr {$line + $n}].$tkPriv(char)]
-    if {[$w compare $new == end] || [$w compare $new == "insert linestart"]} {
+    if {[$w compare $new == end] || [$w compare $new == {insert linestart}]} {
 	set new $i
     }
     set tkPriv(prevPos) $new
@@ -803,13 +832,14 @@ proc tkTextUpDownLine {w n} {
 proc tkTextPrevPara {w pos} {
     set pos [$w index "$pos linestart"]
     while 1 {
-	if {(([$w get "$pos - 1 line"] == "\n") && ([$w get $pos] != "\n"))
-		|| ($pos == "1.0")} {
+      if {(![string compare [$w get "$pos - 1 line"] "\n"]
+              && [string compare [$w get $pos] "\n"])
+              || ![string compare $pos "1.0"]} {
 	    if {[regexp -indices {^[ 	]+(.)} [$w get $pos "$pos lineend"] \
 		    dummy index]} {
 		set pos [$w index "$pos + [lindex $index 0] chars"]
 	    }
-	    if {[$w compare $pos != insert] || ($pos == "1.0")} {
+          if {[$w compare $pos != insert] || ![string compare $pos 1.0]} {
 		return $pos
 	    }
 	}
@@ -828,13 +858,13 @@ proc tkTextPrevPara {w pos} {
 
 proc tkTextNextPara {w start} {
     set pos [$w index "$start linestart + 1 line"]
-    while {[$w get $pos] != "\n"} {
+    while {[string compare [$w get $pos] "\n"]} {
 	if {[$w compare $pos == end]} {
 	    return [$w index "end - 1c"]
 	}
 	set pos [$w index "$pos + 1 line"]
     }
-    while {[$w get $pos] == "\n"} {
+    while {![string compare [$w get $pos] "\n"]} {
 	set pos [$w index "$pos + 1 line"]
 	if {[$w compare $pos == end]} {
 	    return [$w index "end - 1c"]
@@ -862,7 +892,7 @@ proc tkTextNextPara {w start} {
 proc tkTextScrollPages {w count} {
     set bbox [$w bbox insert]
     $w yview scroll $count pages
-    if {$bbox == ""} {
+    if {![string compare $bbox ""]} {
 	return [$w index @[expr {[winfo height $w]/2}],0]
     }
     return [$w index @[lindex $bbox 0],[lindex $bbox 1]]
@@ -932,7 +962,7 @@ proc tk_textCut w {
 proc tk_textPaste w {
     global tcl_platform
     catch {
-	if {"$tcl_platform(platform)" != "unix"} {
+      if {[string compare $tcl_platform(platform) "unix"]} {
 	    catch {
 		$w delete sel.first sel.last
 	    }
@@ -951,7 +981,7 @@ proc tk_textPaste w {
 # w -		The text window in which the cursor is to move.
 # start -	Position at which to start search.
 
-if {$tcl_platform(platform) == "windows"}  {
+if {![string compare $tcl_platform(platform) "windows"]}  {
     proc tkTextNextWord {w start} {
 	tkTextNextPos $w [tkTextNextPos $w $start tcl_endOfWord] \
 	    tcl_startOfNextWord
