@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacDraw.c,v 1.2 1998/09/14 18:23:35 stanton Exp $
+ * RCS: @(#) $Id: tkMacDraw.c,v 1.4 1999/04/16 01:51:30 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -988,9 +988,9 @@ TkMacSetUpGraphicsPort(
 	 * the Mac (or for Windows), please let me know.
 	 *
 	 *	Jan Nijtmans
-	 *	NICI (Nijmegen Institute of Cognition and Information)
-	 *	email: Jan.Nijtmans@cmg.nl (work)
-	 *	       Jan.Nijtmans@wxs.nl (private)
+	 *	CMG Arnhem, B.V.
+	 *	email: Jan.Nijtmans@wxs.nl (private)
+	 *	       Jan.Nijtmans@wxs.nl (work)
 	 *	url:   http://home.wxs.nl/~nijtmans/
 	 */
     }
