@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacXStubs.c 1.87 97/11/20 18:35:29
+ * RCS: @(#) $Id: tkMacXStubs.c,v 1.6 1999/04/16 01:51:33 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -575,6 +575,47 @@ XForceScreenSaver(
      */
     display->request++;
 }
+
+void
+Tk_FreeXId (
+    Display *display,
+    XID xid)
+{
+    /* no-op function needed for stubs implementation. */
+}
+
+void
+Tk_3DHorizontalBevel (
+    Tk_Window tkwin,
+    Drawable d,
+    Tk_3DBorder b,
+    int x,
+    int y,
+    int width,
+    int height,
+    int leftIn,
+    int rightIn,
+    int topBevel,
+    int relief)
+{
+    /* no-op function needed for stubs implementation. */
+}
+
+void
+Tk_3DVerticalBevel (
+    Tk_Window tkwin,
+    Drawable d,
+    Tk_3DBorder b,
+    int x,
+    int y,
+    int width,
+    int height,
+    int leftBevel,
+    int relief)
+{
+    /* no-op function needed for stubs implementation. */
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -601,7 +642,8 @@ TkGetServerInfo(
     Tk_Window tkwin)		/* Token for window;  this selects a
 				 * particular display and server. */
 {
-    char buffer[50], buffer2[50];
+    char buffer[8 + TCL_INTEGER_SPACE * 2];
+    char buffer2[TCL_INTEGER_SPACE];
 
     sprintf(buffer, "X%dR%d ", ProtocolVersion(Tk_Display(tkwin)),
 	    ProtocolRevision(Tk_Display(tkwin)));
