@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinDraw.c,v 1.2 1998/09/14 18:24:00 stanton Exp $
+ * RCS: @(#) $Id: tkWinDraw.c,v 1.4 1999/04/16 01:51:51 stanton Exp $
  */
 
 #include "tkWinInt.h"
@@ -106,11 +106,16 @@ static int bltModes[] = {
 typedef BOOL (CALLBACK *WinDrawFunc) _ANSI_ARGS_((HDC dc,
 			    CONST POINT* points, int npoints));
 
+typedef struct ThreadSpecificData {
+    POINT *winPoints;    /* Array of points that is reused. */
+    int nWinPoints;	/* Current size of point array. */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
+
 /*
  * Forward declarations for procedures defined in this file:
  */
 
-static HPEN		SetUpGraphicsPort _ANSI_ARGS_((GC gc));
 static POINT *		ConvertPoints _ANSI_ARGS_((XPoint *points, int npoints,
 			    int mode, RECT *bbox));
 static void		DrawOrFillArc _ANSI_ARGS_((Display *display,
@@ -120,6 +125,7 @@ static void		DrawOrFillArc _ANSI_ARGS_((Display *display,
 static void		RenderObject _ANSI_ARGS_((HDC dc, GC gc,
 			    XPoint* points, int npoints, int mode, HPEN pen,
 			    WinDrawFunc func));
+static HPEN		SetUpGraphicsPort _ANSI_ARGS_((GC gc));
 
 /*
  *----------------------------------------------------------------------
@@ -213,7 +219,8 @@ TkWinReleaseDrawableDC(d, dc, state)
  *	Returns the converted array of POINTs.
  *
  * Side effects:
- *	Allocates a block of memory that should not be freed.
+ *	Allocates a block of memory in thread local storage that 
+ *      should not be freed.
  *
  *----------------------------------------------------------------------
  */
@@ -225,8 +232,8 @@ ConvertPoints(points, npoints, mode, bbox)
     int mode;			/* CoordModeOrigin or CoordModePrevious. */
     RECT *bbox;			/* Bounding box of points. */
 {
-    static POINT *winPoints = NULL; /* Array of points that is reused. */
-    static int nWinPoints = -1;	    /* Current size of point array. */
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     int i;
 
     /*
@@ -234,16 +241,16 @@ ConvertPoints(points, npoints, mode, bbox)
      * we reuse the last array if it is large enough.
      */
 
-    if (npoints > nWinPoints) {
-	if (winPoints != NULL) {
-	    ckfree((char *) winPoints);
+    if (npoints > tsdPtr->nWinPoints) {
+	if (tsdPtr->winPoints != NULL) {
+	    ckfree((char *) tsdPtr->winPoints);
 	}
-	winPoints = (POINT *) ckalloc(sizeof(POINT) * npoints);
-	if (winPoints == NULL) {
-	    nWinPoints = -1;
+	tsdPtr->winPoints = (POINT *) ckalloc(sizeof(POINT) * npoints);
+	if (tsdPtr->winPoints == NULL) {
+	    tsdPtr->nWinPoints = -1;
 	    return NULL;
 	}
-	nWinPoints = npoints;
+	tsdPtr->nWinPoints = npoints;
     }
 
     bbox->left = bbox->right = points[0].x;
@@ -251,26 +258,26 @@ ConvertPoints(points, npoints, mode, bbox)
     
     if (mode == CoordModeOrigin) {
 	for (i = 0; i < npoints; i++) {
-	    winPoints[i].x = points[i].x;
-	    winPoints[i].y = points[i].y;
-	    bbox->left = MIN(bbox->left, winPoints[i].x);
-	    bbox->right = MAX(bbox->right, winPoints[i].x);
-	    bbox->top = MIN(bbox->top, winPoints[i].y);
-	    bbox->bottom = MAX(bbox->bottom, winPoints[i].y);
+	    tsdPtr->winPoints[i].x = points[i].x;
+	    tsdPtr->winPoints[i].y = points[i].y;
+	    bbox->left = MIN(bbox->left, tsdPtr->winPoints[i].x);
+	    bbox->right = MAX(bbox->right, tsdPtr->winPoints[i].x);
+	    bbox->top = MIN(bbox->top, tsdPtr->winPoints[i].y);
+	    bbox->bottom = MAX(bbox->bottom, tsdPtr->winPoints[i].y);
 	}
     } else {
-	winPoints[0].x = points[0].x;
-	winPoints[0].y = points[0].y;
+	tsdPtr->winPoints[0].x = points[0].x;
+	tsdPtr->winPoints[0].y = points[0].y;
 	for (i = 1; i < npoints; i++) {
-	    winPoints[i].x = winPoints[i-1].x + points[i].x;
-	    winPoints[i].y = winPoints[i-1].y + points[i].y;
-	    bbox->left = MIN(bbox->left, winPoints[i].x);
-	    bbox->right = MAX(bbox->right, winPoints[i].x);
-	    bbox->top = MIN(bbox->top, winPoints[i].y);
-	    bbox->bottom = MAX(bbox->bottom, winPoints[i].y);
+	    tsdPtr->winPoints[i].x = tsdPtr->winPoints[i-1].x + points[i].x;
+	    tsdPtr->winPoints[i].y = tsdPtr->winPoints[i-1].y + points[i].y;
+	    bbox->left = MIN(bbox->left, tsdPtr->winPoints[i].x);
+	    bbox->right = MAX(bbox->right, tsdPtr->winPoints[i].x);
+	    bbox->top = MIN(bbox->top, tsdPtr->winPoints[i].y);
+	    bbox->bottom = MAX(bbox->bottom, tsdPtr->winPoints[i].y);
 	}
     }
-    return winPoints;
+    return tsdPtr->winPoints;
 }
 
 /*
@@ -1170,16 +1177,6 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
  *----------------------------------------------------------------------
  */
 
-#ifndef TkWinCreatePen
-#define TkWinCreatePen(gc, style, width, foreground) \
-	CreatePen(style, width, foreground)
-#endif
-
-#ifndef TkWinExtCreatePen
-#define TkWinExtCreatePen(gc, style, width, brush, count, lp) \
-	ExtCreatePen(style, width, brush, count, lp)
-#endif
-
 static HPEN
 SetUpGraphicsPort(gc)
     GC gc;
@@ -1217,7 +1214,7 @@ SetUpGraphicsPort(gc)
 	if (gc->line_width > 1) {
 	    style = PS_SOLID;
 	}
-	return TkWinCreatePen(gc, style, gc->line_width, gc->foreground);
+	return CreatePen(style, gc->line_width, gc->foreground);
     } else {
 	LOGBRUSH lb;
 
@@ -1249,7 +1246,7 @@ SetUpGraphicsPort(gc)
 		style |= PS_JOIN_BEVEL; 
 		break;
 	}
-	return TkWinExtCreatePen(gc, style, gc->line_width, &lb, 0, NULL);
+	return ExtCreatePen(style, gc->line_width, &lb, 0, NULL);
     }
 }
 
